globals 
[ 
  datalist
  catchlist
  BootstrapEstimates 
  temp
  sample
  statistic
  GettingSample
  delt
  FishMax
  Mx
  Mn
  CILow
  CIHigh
  Len
  tmp
  StatisticEstimate
  MeasurementLatency
]

breed [fishes fish]
breed [boats boat] 
breed [banners banner]


fishes-own [ value bodysize ]
boats-own [ currentsample ]

to attach-banner [x]  ;; circle procedure
  hatch-banners 1 [
    set label-color white
    set size 0
    set label x
    create-link-from myself [
      tie
      hide-link
    ]
  ]
end

to change-banner [x] ;; change value of chip
  ask out-link-neighbors [ set label x ]
end

to randomize
  ifelse ( NumberOfFish < 20 )
    [  set datalist n-values NumberOfFish [ precision ( random-exponential TargetMean ) 0 ] ]
    [  set datalist n-values NumberOfFish [ precision ( random-exponential TargetMean ) 3 ] ]
end 

to initialize
  clear-turtles
  clear-all-plots
  reset-ticks
  set-default-shape fishes "fish"
  set-default-shape boats  "boat"
  ask patches [ set pcolor blue ]
  set MeasurementLatency 5 
  
  create-boats 1 [ 
      set color grey 
      set heading 90
      set size 7
      attach-banner ""
    ]
  
  set FishMax max ( datalist )
  foreach datalist 
    [
      create-fishes 1 [
         ifelse( ?1 < 0 ) 
           [ set color red ]
           [ set color green ]
         setxy random-xcor random-ycor
         set heading random 360
         
         set value ?1
         set label-color grey + 2
         set bodysize precision ( abs ( value / FishMax ) * 2 ) 3 
         
         ifelse( Values )
          [
            set size 0 
            set label value 
          ] 
          [ 
            set size bodysize
            set label "" 
          ]    
      ]     
    ]
 
  ask boats [ set sample [] ]  
  set BootstrapEstimates []
  set GettingSample false 
  set CILow 0 
  set CIHigh 0
end 

to SetUp
  Randomize 
  initialize
end


to Go
  tick
  if( ticks mod MeasurementLatency = 0 ) 
    [
      ask boats  
        [ 
          set temp one-of fishes-here 
          ifelse( temp != nobody ) 
            [ 
               set sample lput [ value ] of temp sample 
               set currentsample [ value ] of temp 
               change-banner currentsample
               set GettingSample false 
            ]
            [ set GettingSample true ]
          if( ( length sample ) >= SampleSize ) 
            [ 
              ifelse( Target_Statistic = "median") 
                [ set statistic median sample ]
                [ set statistic mean sample ] 
              set statistic precision statistic 4
              set BootstrapEstimates lput statistic BootstrapEstimates
              set sample []
            ]
        ]
    ]
    
  if( ticks mod MeasurementLatency = floor( 0.5 * MeasurementLatency ) ) 
    [ ask boats [ change-banner "" ] ]
  
  ask fishes 
    [ 
      rt random 10
      lt random 10 
      fd 1
      ifelse( Values )
        [
          set size 0 
          set label value 
        ] 
        [ 
          set size bodysize
          set label "" 
        ]
    ]
    UpdatePlots
end

to UpdatePlots 
  if( length BootstrapEstimates > 1 ) 
    [ 
      set BootstrapEstimates sort BootstrapEstimates
      set Len length BootstrapEstimates 
      set StatisticEstimate precision ( mean BootstrapEstimates ) 4
      set Mx max BootstrapEstimates
      set Mn min BootstrapEstimates
      set delt ( Mx - Mn ) / 20
      
      if( delt > 0 ) 
        [  
          set-plot-pen-interval delt 
          set CILow  item floor   ( 0.025 * Len ) BootstrapEstimates
          ifelse( ceiling ( 0.975 * Len ) = Len ) 
            [ set CIHigh item (Len - 1 ) BootstrapEstimates ]
            [ set CIHigh item ceiling ( 0.975 * Len )  BootstrapEstimates ]
          set CILow  precision CILow  4
          set CIHigh precision CIHigh 4
          set-plot-x-range precision  Mn 4  precision  Mx  4  
          histogram BootstrapEstimates
        ]
    ]
end 



to LoadData
  set datalist read-from-string ( word ( word "[" Data ) "]" ) 
  set Values true 
  initialize
end

to ShowData
  set tmp ""
  ask fishes [ set tmp word tmp word value " " ]
  set Data tmp
end
@#$#@#$#@
GRAPHICS-WINDOW
210
10
652
473
4
4
48.0
1
16
1
1
1
0
1
1
1
-4
4
-4
4
0
0
1
ticks
30.0

INPUTBOX
5
585
650
645
Data
1 2 3 4 5 6 7 8 9
1
0
String

TEXTBOX
10
665
465
690
Enter numbers separated by spaces into Data.  Then press 'Load Data'.
14
0.0
1

BUTTON
455
655
552
688
Load Data
LoadData
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
5
10
60
43
Setup
setup
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
70
10
125
43
Go
Go
T
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

SLIDER
5
55
205
88
NumberOfFish
NumberOfFish
2
100
50
1
1
NIL
HORIZONTAL

PLOT
5
370
205
525
BootstrapHistogram
NIL
NIL
0.0
10.0
0.0
10.0
true
false
"" ""
PENS
"BSTP" 1.0 1 -7500403 true "" ""

MONITOR
5
530
650
583
Current sample
sample
17
1
13

CHOOSER
95
125
200
170
Target_Statistic
Target_Statistic
"mean" "median"
0

MONITOR
80
310
140
363
Low
CILow
17
1
13

MONITOR
140
310
205
363
High
CIHigh
17
1
13

TEXTBOX
10
310
85
365
95% Confidence Interval
14
0.0
1

SLIDER
5
90
205
123
SampleSize
SampleSize
5
50
10
5
1
NIL
HORIZONTAL

SWITCH
5
190
95
223
Values
Values
1
1
-1000

MONITOR
5
250
205
303
Bootstrap Estimate of Statistic
StatisticEstimate
17
1
13

INPUTBOX
5
125
85
185
TargetMean
10
1
0
Number

MONITOR
210
475
650
528
Boot Strap Estimates
BootstrapEstimates
17
1
13

TEXTBOX
115
175
185
230
Bootstrap Estimates Below
14
0.0
1

BUTTON
565
655
650
688
Show Data
ShowData
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

@#$#@#$#@
## WHAT IS IT?

Bootstrapping in statistics is a simple idea that nonetheless can be difficult for some to understand.  This applet provides a visual illustration of the bootstrap process.  In particular, a lake has many sizes of fish, including possibly a few "whoppers" scattered about.  The length of the fish in the lake is a single dimensional dataset, but estimating the average length of a fish in the lake is non-trivial. The "whoppers", if they exist, are outliers.  The distribution of the fish is unknown. Consequently, there is good reason to create a bootstrap estimate of the mean length of a fish in the lake, along with its associated variance and confidence intervals.  

## HOW IT WORKS

As the fish pass under the boat, they are caught, their length is measured, and then they are released.  After a five tick delay to allow the fish population to redistribute, the process is repeated, possibly catching the same fish again (sampling with replacement).  Once a sufficiently large sample is capture, the statistic (mean or median) is applied and the estimate is added to a list of BootStrapEstimates.  The BootstrapEstimates is an empirical distribution for the statistic which also allows the estimate of a confidence interval for the length.  

However, the length of a fish is a metaphor for any univariate data.  To illustrate this, switching on Values shows the numerical value in place of the fish.  Moreover, univariate data can be loaded as space separated values and can also be produced via 'Show Data' in the Data box (negative numbers produce 'red' fish).    

## HOW TO USE IT

'Setup' generates an exponentially distributed random sample with a mean of 'TargetMean' of size 'NumberOfFish'.  'SampleSize' indicates the size of the samples used to estimate the statistic.  'Load Data' loads the space separated values in the 'Data' box and sets the 'Values' switch to on.  'Target_Statistic' chooses the desired statistic.  'Go' begins the simulation, in which samples are loaded into the 'Current Sample' box until they reach size 'SampleSize', at which point an estimate of the statistic is produced. The estimate is added to the list in the 'Bootstrap Estimates' box. The estimate, a histogram of 'Bootstrap Estimates', and the 95% confidence interval for the empirical distribution are subsequently produced.   

## THINGS TO NOTICE

Sampling with replacement corresponds to a previously captured fish once again swimming beneath the boat and being captured.  Varying the sample size improves the confidence interval.  
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

boat
true
0
Polygon -6459832 true false 90 255 105 270 180 270 195 255 90 255 150 255
Rectangle -6459832 true false 90 90 120 255
Rectangle -6459832 true false 165 90 195 255
Rectangle -6459832 true false 90 225 180 255
Rectangle -7500403 true true 135 135 150 210
Polygon -6459832 true false 90 120 195 120 195 90 150 15 135 15 90 90

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

circle
false
0
Circle -7500403 true true 0 0 300

circle 2
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

cylinder
false
0
Circle -7500403 true true 0 0 300

dot
false
0
Circle -7500403 true true 90 90 120

face happy
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
true
0
Polygon -1 true false 131 256 87 279 86 285 120 300 150 285 180 300 214 287 212 280 166 255
Polygon -1 true false 195 165 235 181 218 205 210 224 204 254 165 240
Polygon -1 true false 45 225 77 217 103 229 114 214 78 134 60 165
Polygon -7500403 true true 136 270 77 149 81 74 119 20 146 8 160 8 170 13 195 30 210 105 212 149 166 270
Circle -16777216 true false 106 55 30

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7500403 true 150 0 150 300

line half
true
0
Line -7500403 true 150 0 150 150

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

sheep
false
15
Circle -1 true true 203 65 88
Circle -1 true true 70 65 162
Circle -1 true true 150 105 120
Polygon -7500403 true false 218 120 240 165 255 165 278 120
Circle -7500403 true false 214 72 67
Rectangle -1 true true 164 223 179 298
Polygon -1 true true 45 285 30 285 30 240 15 195 45 210
Circle -1 true true 3 83 150
Rectangle -1 true true 65 221 80 296
Polygon -1 true true 195 285 210 285 210 240 240 210 195 210
Polygon -7500403 true false 276 85 285 105 302 99 294 83
Polygon -7500403 true false 219 85 210 105 193 99 201 83

square
false
0
Rectangle -7500403 true true 30 30 270 270

square 2
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7500403 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

triangle
false
0
Polygon -7500403 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7500403 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269

wolf
false
0
Polygon -16777216 true false 253 133 245 131 245 133
Polygon -7500403 true true 2 194 13 197 30 191 38 193 38 205 20 226 20 257 27 265 38 266 40 260 31 253 31 230 60 206 68 198 75 209 66 228 65 243 82 261 84 268 100 267 103 261 77 239 79 231 100 207 98 196 119 201 143 202 160 195 166 210 172 213 173 238 167 251 160 248 154 265 169 264 178 247 186 240 198 260 200 271 217 271 219 262 207 258 195 230 192 198 210 184 227 164 242 144 259 145 284 151 277 141 293 140 299 134 297 127 273 119 270 105
Polygon -7500403 true true -1 195 14 180 36 166 40 153 53 140 82 131 134 133 159 126 188 115 227 108 236 102 238 98 268 86 269 92 281 87 269 103 269 113

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 5.0.3
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
default
0.0
-0.2 0 1.0 0.0
0.0 1 1.0 0.0
0.2 0 1.0 0.0
link direction
true
0
Line -7500403 true 150 150 90 180
Line -7500403 true 150 150 210 180

@#$#@#$#@
1
@#$#@#$#@
